#
# Borrowed from: http://www.theresearchkitchen.com/archives/679
#
euler = function(dy.dx=function(x,y){}, h=1E-7, y0=1, start=0, end=1){
    nsteps = (end-start)/h
    xs = numeric(nsteps+1)
    ys = numeric(nsteps+1)
    xs[1] = start
    ys[1] = y0
    for (i in 1:nsteps){
        x = start + (i-1)*h
        ys[i+1] = ys[i] + h*dy.dx(x,ys[i])
        xs[i+1] = x + h
    }
    list( xs=xs, ys=ys )
}

#
# An example of how to use use this function is:
#
# dy.dx = function(x,y) { 3*x - y + 8 }
#
# res = euler(dy.dx, start=0, end=0.5, h=0.1, y0=3)
# res$ys
#
# [1] 3.00000 3.50000 3.98000 4.44200 4.88780 5.31902
#


rounded_euler = function(dy.dx=function(x,y){}, h=1E-7, y0=1, start=0, end=1, n_digits=3){
    #
    # Here we round the computations so that they only have n_digits.
    # 
    # This simulates using floating point arithmetic with a fixed word size
    # and demonstrates the affect of rounding errors.
    # 
    nsteps = round((end-start)/h, n_digits)
    xs = numeric(nsteps+1)
    ys = numeric(nsteps+1)
    xs[1] = round(start, n_digits)
    ys[1] = round(y0, n_digits)
    for (i in 1:nsteps){
        x = round(start + (i-1)*h, n_digits)
        ys[i+1] = round(ys[i] + h*dy.dx(x,ys[i]), n_digits)
        xs[i+1] = round(x + h, n_digits)
    }
    list( xs=xs, ys=ys )
}

