library(deSolve)

dudt = function(t, state, parameters){
    with(as.list(c(state, parameters)),{
        du = uprime
        duprime = -u - u^3/5 + cos(omega*t)
        list(c(du, duprime))
    })
}
    
t = seq( 0, 50, length.out=1000 )
    
state = c( u=0, uprime=0 )
    
# Part (a):
#
parms = c( omega=1 )

res = ode(y=state, times=t, fun=dudt, parms=parms)
ts = res[,'time']
us = res[,'u']

#postscript("../../WriteUp/Graphics/Chapter3/chap_3_sect_8_prob_24_part_a_plot.eps", onefile=FALSE, horizontal=FALSE)
plot( ts, us, col='black', type='l', xlab='t', ylab='u(t)')
grid()
#dev.off()

# Part (b):
#
omegas = c(0.51, 1.0, 2.0)
US = c()
for( og in omegas ){
    parms = c( omega=og )
    res = ode(y=state, times=t, fun=dudt, parms=parms)
    ts = res[,'time']
    us = res[,'u']
    
    US = cbind(US, us)
}

#postscript("../../WriteUp/Graphics/Chapter3/chap_3_sect_8_prob_24_part_b_plot.eps", onefile=FALSE, horizontal=FALSE)
matplot(ts, US, type='l', lty=1, xlab='t', ylab='u(t)', ylim=c(-3.5, +4.5))
grid()
legend('topleft', sprintf('omega= %.1f', omegas), lty=1, col=1:length(omegas))
#dev.off()

