library(deSolve)

source('./utils.R')

dudt = function(t, state, parameters){
    with(as.list(c(state, parameters)),{
        du = uprime
        duprime = -uprime/5 - u - u^3/5 + cos(omega*t)
        list(c(du, duprime))
    })
}

t = seq( 0, 50, length.out=1000 )

state = c( u=0, uprime=0 )

# Part (a):
#
omegas = c(0.51, 1.0, 2.0)

US = c()
for( og in omegas ){
    parms = c( omega=og )
    res = ode(y=state, times=t, fun=dudt, parms=parms)
    ts = res[,'time']
    us = res[,'u']

    US = cbind(US, us)
}

##postscript("../../WriteUp/Graphics/Chapter3/chap_3_sect_8_prob_25_parts_a_N_b_plots.eps", onefile=FALSE, horizontal=FALSE)

par(mfrow=c(1, 2))
matplot(ts, US, type='l', lty=1, xlab='t', ylab='u(t)', ylim=c(-3.5, +4.5))
grid()
legend('topleft', sprintf('omega= %.1f', omegas), lty=1, col=1:length(omegas))

# Part (b):
#
omegas = seq(0.5, 2.0, length.out=30)

US = c()
for( og in omegas ){
    parms = c( omega=og )
    res = ode(y=state, times=t, fun=dudt, parms=parms)
    ts = res[,'time']
    us = res[,'u']

    US = cbind(US, us)
}

Rs = as.double(sapply(data.frame(US), my_amplitude))

omega_argmax = omegas[which.max(Rs)]
print(sprintf('maximum amplitude omega: %.3f', omega_argmax))

R = function(og){
    denom = sqrt( (1-og^2)^2 + og^2/25 )
    1/denom
}
Rs_linear = R(omegas) ## the linear approximation

plot(omegas, Rs, ylim=c(0.25, 5.0), type='b', pch=20, col='black', xlab='omega', ylab='amplitude')
lines(omegas, Rs_linear, type='l', col='blue')
abline(v=omega_argmax, col='red')
grid()
par(mfrow=c(1, 1))

##dev.off()
