if( !require('phaseR') ){
    install.packages('phaseR')
}
library(phaseR)

my_jacobian = function(x, y) {
    # returns the Jacobian at the point (x, y)
    #
    J = matrix(c(-1 - y/2, 2+2*y-x/2, 1-y-x, 2-x), nrow=2, ncol=2, byrow=TRUE)
}

# The critical points of this system:
CP = data.frame(x=c(0, 2, 2, 4), y=c(0, -2, 1, -2))

As = mapply(my_jacobian, CP$x, CP$y, SIMPLIFY=FALSE)

for (ii in 1:length(As)){
    es = eigen(As[[ii]])
    print(sprintf('CP: x= %f; y= %f', CP$x[ii], CP$y[ii]))
    print('Jacobian=')
    print(As[[ii]])
    print('eigenvalues=')
    print(es$values)
}

my_yprime = function(t, y, parameters) {
    xt = y[1]
    yt = y[2]
    dy = rep(NA, length(y))
    dy[1] = (2+yt)*(yt-0.5*xt)
    dy[2] = (2-xt)*(y+0.5*xt)
    list(dy)
}

diff_eq_params = list()

#postscript('../../WriteUp/Graphics/Chapter9/chap_9_sect_3_prob_9_plot.eps', onefile=FALSE, horizontal=FALSE)

t.end = 5.0
L = 6
flowField(my_yprime, x.lim = c(-L, +L), y.lim = c(-L, +L), parameters = diff_eq_params, points = 21, add = FALSE, xlab='x', ylab='y')

# Plot some trajectories with initial condition near the critical points:
#
trajectory(my_yprime, y0 = c(-1.25, 1.5), t.end = t.end, parameters = diff_eq_params, col='black', pch=19)
trajectory(my_yprime, y0 = c(-1, -2), t.end = t.end, parameters = diff_eq_params, col='blue', pch=19)
trajectory(my_yprime, y0 = c(0.1, 1.25), t.end = t.end, parameters = diff_eq_params, col='red', pch=19)
trajectory(my_yprime, y0 = c(0.9, 0.1), t.end = t.end, parameters = diff_eq_params, col='green', pch=19)

#dev.off()

