if( !require('phaseR') ){
    install.packages('phaseR')
}
library(phaseR)
library(deSolve)

source('../Chapter3/utils.R')

my_yprime = function(t, y, parameters) {
    xt = y[1]
    yt = y[2]
    dy = rep(NA, length(y))
    dy[1] = xt - 0.5*xt*yt
    dy[2] = -0.75*yt + 0.25*xt*yt
    list(dy)
}

# Parts (c):
#
ts = seq(0, 10, length.out=500)

res = ode(y=c(5, 2), times=ts, fun=my_yprime, parms=list()) # guess the initial conditions the book uses for its plot
xs = res[, 2] - mean(res[, 2]) # the numerical function x(t) (demeaned)
ys = res[, 3] - mean(res[, 3]) # the numerical function y(t) (demeaned)
Ax = my_amplitude(xs)
Ay = my_amplitude(ys)
print(Ax/Ay)


# Part (d):
#
a = 1
alpha = 0.5
c = 0.75
gamma = 0.25

exact_ratio = sqrt(3)

x0 = c/gamma # the critical point
y0 = a/alpha

#postscript('../../WriteUp/Graphics/Chapter9/chap_9_sect_5_prob_7_plot.eps', onefile=FALSE, horizontal=FALSE)

As = seq(1.01, 2, length.out=10) # move the initial conditions (x, y) from the critical point (x0, y0) by multiplying by this factor
x_amplitude_estimates = c()
y_amplitude_estimates = c()
for( a in As ){
    res = ode(y=a*c(x0, y0), times=ts, fun=my_yprime, parms=list())
    xs = res[, 2] # the numerical function x(t)
    xs = xs - mean(xs) # (demeaned)
    ys = res[, 3] # the numerical function y(t)
    ys = ys - mean(ys) # (demeaned)

    Ax = my_amplitude(xs)
    Ay = my_amplitude(ys)

    x_amplitude_estimates = c(x_amplitude_estimates, Ax)
    y_amplitude_estimates = c(y_amplitude_estimates, Ay)
}
x_over_y_amplitude_estimates = x_amplitude_estimates/y_amplitude_estimates

plot(As, x_over_y_amplitude_estimates, type='b', lty=1, lwd=2, pch=19, xlab='expansion factor from critical point', ylab='amplitude ratio Ax/Ay (est)')
abline(h=exact_ratio, col='green')
grid()

#dev.off()
